$ifndef __ADC_INC
$define __ADC_INC
'
' ADC routines for Amicus18
'
$Include ADCdefs.Inc

'-------------------------------------------------------------------------
' *** Interrupt enable/disable ***
'-------------------------------------------------------------------------
$define ADC_IntEnable()  '
    PIR1bits_ADIF = 0    '
    INTCONbits_PEIE = 1  '
    PIE1bits_ADIE = 1

'-------------------------------------------------------------------------
$define ADC_IntDisable() PIE1bits_ADIE = 0

'-------------------------------------------------------------------------
$define ConvertADC() ADCON0bits_GO_DONE = 1


'$ifndef _Amicus18_Rev2 ' *** Code for Revision 1 ***
'-------------------------------------------------------------------------
' Open the ADC
'-------------------------------------------------------------------------
$define OpenADC(pConfig,pConfig2,pPortConfig) _OpenADC pConfig, pConfig2, pPortConfig

_OpenADC Macro- pConfig, pConfig2, pPortConfig
    #if(Prm_Count != 3)
        #error "OpenADC must have 3 Parameters (pConfig , pConfig2, pPortConfig)"
        Exitm
    #endif
    #if((Prm_1 != Num8) && (Prm_1 != Num16))
        #error "OpenADC Parameters must be constant values"
        Exitm
    #endif
    #if((Prm_2 != Num8) && (Prm_2 != Num16))
        #error "OpenADC Parameters must be constant values"
        Exitm
    #endif
    #if((Prm_3 != Num8) && (Prm_3 != Num16))
        #error "OpenADC Parameters must be constant values"
        Exitm
    #endif
    Num_SFR ((pConfig2 >> 1) & 0x3C),ADCON0
    Num_SFR ((pConfig2 << 4) & 0x30),ADCON1
    Num_SFR (pConfig & 128 | ((pConfig >> 4) & 0x07) | ((pConfig << 2) & 0x38)),ADCON2

    Num_SFR pPortConfig,ANSEL

    #if((pPortConfig >> 8) == 0)
        Clrf ANSELH
    #else
        Num_SFR (pPortConfig >> 8),ANSELH
    #endif
    ADCON0bits_ADON = 1
    '
    ' Dummy read, as per errata workaround
    '
    ADCON0bits_GO_DONE = 1
    #ifdef WatchDog_Req
        Clrwdt
    #endif
    Btfsc ADCON0bits_GO_DONE
    Bra($ - 2)
Endm

'-------------------------------------------------------------------------
' *** CloseADC Macro ***
'-------------------------------------------------------------------------

$define CloseADC()      '
    ADCON0bits_ADON = 0 '
    PIE1bits_ADIE = 0

'-------------------------------------------------------------------------
' *** SelChanConvADC Macro ***
'-------------------------------------------------------------------------

$define SelChanConvADC(pCH) _SelChanConvADC pCH

_SelChanConvADC Macro- pChannel
    #if(Prm_Count != 1)
        #error "SelChanConvADC must have 1 Parameter"
        Exitm
    #endif
    #if((Prm_1 != Num8) && (Prm_1 != Num16))
        #error "SelChanConvADC Parameter must be constant value"
        Exitm
    #endif
    Movlw 0xC3
    Andwf ADCON0,W
    Iorlw ((pChannel >> 1) & 0x3C)
    Movwf ADCON0
    ADCON0bits_GO_DONE = 1
Endm

'-------------------------------------------------------------------------
' *** SetChanADC Macro ***
'-------------------------------------------------------------------------

$define SetChanADC(pCH) _SetChanADC pCH

_SetChanADC Macro- pChannel
    #if(Prm_Count != 1)
        #error "SetChanADC must have 1 Parameter"
        Exitm
    #endif
    #if((Prm_1 != Num8) && (Prm_1 != Num16))
        #error "SetChanADC Parameter must be constant value"
        Exitm
    #endif
    Num_SFR ((pChannel >> 1) & 0x3C),ADCON0
    Movlw 0xC3
    Andwf ADCON0,W
    Iorwf ADCON0,F
Endm
'-------------------------------------------------------------------------
' *** BusyADC Macro ***
'-------------------------------------------------------------------------
$define BusyADC() _BusyADC

_BusyADC Macro- \ Byte

    #if(BusyADC_Return == 1)
        Movlw 0
        Btfsc ADCON0bits_GO_DONE
        Movlw 1
        #if(Return_Type == Byte)
            wreg_byte Return_Var
        #endif
        #if(Return_Type == Word)
            Wreg_Word Return_Var
        #endif
        #if(Return_Type == Dword)
            Wreg_Dword Return_Var
        #endif
        #if(Return_Type == Bit)
            Wreg_Bit Return_Var, Return_BitNum
        #endif
    #endif
Endm
'-------------------------------------------------------------------------
' *** ReadADC Macro ***
'-------------------------------------------------------------------------
$define ReadADC(pCH) _ReadADC pCH

_ReadADC Macro- pChannel \ Word
    #if(Prm_Count == 0)
        ADCON0bits_GO_DONE = 1
        #ifdef WatchDog_Req
            Clrwdt
        #endif
        Btfsc ADCON0bits_GO_DONE
        Bra($ - 2)
        
        #if(_ReadADC_Return == 1)
            #if(Return_Type == Bit)
                Byte_Bit ADRESH, Return_Var, Return_BitNum
            #endif
            #if(Return_Type == Byte)
                Byte_Byte ADRESH,Return_Var
            #endif
            #if(Return_Type == Word)
                #if(Return_Var != ADRESL)
                    Byte_Byte ADRESL,Return_Var
                    Byte_Byte ADRESH,Return_Var + 1
                #endif
            #endif
            #if(Return_Type == Dword)
                Byte_Byte ADRESL,Return_Var
                Byte_Byte ADRESH,Return_Var + 1
                Num_Word 0,Return_Var + 2
            #endif
        #endif
        Exitm
    #endif

    #if(Prm_Count > 1)
        #error "ReadADC must have only 1 Parameter"
        Exitm
    #endif

    #if((Prm_1 != Num8) && (Prm_1 != Num16))
        #error "ReadADC Parameter must be constant value"
        Exitm
    #endif
    Movlw $C3
    Andwf ADCON0,W
    Iorlw ((pChannel >> 1) & 0x3C)
    Movwf ADCON0
    ADCON0bits_GO_DONE = 1
    #ifdef WatchDog_Req
        Clrwdt
    #endif
    Btfsc ADCON0bits_GO_DONE
    Bra($ - 2)
    #if(_ReadADC_Return == 1)
        #if(Return_Type == Bit)
                Byte_Bit ADRESH, Return_Var, Return_BitNum
            #endif
        #if(Return_Type == Byte)
            Byte_Byte ADRESH,Return_Var
        #endif
        #if(Return_Type == Word)
            #if(Return_Var != ADRESL)
                Byte_Byte ADRESL,Return_Var
                Byte_Byte ADRESH,Return_Var + 1
            #endif
        #endif
        #if(Return_Type == Dword)
            Byte_Byte ADRESL,Return_Var
            Byte_Byte ADRESH,Return_Var + 1
            Num_Word 0,Return_Var + 2
        #endif
    #endif
Endm
'***************************************************************
' Amicus18 Revision 2 macros
'***************************************************************
$else   ' ** Code for Amicus Revision 2 **

' ********** NOT FINISHED ******
$define OpenADC(pConfig,pConfig2,pConfig3,pPortConfig) _OpenADC pConfig, pConfig2, pConfig3, pPortConfig

_OpenADC Macro- pConfig, pConfig2, pConfig3, pPortConfig
    #if(Prm_Count != 4)
        #error "OpenADC for revision 2 must have 4 Parameters (pConfig, pConfig2, pConfig3, pPortConfig)"
        Exitm
    #endif
    #if((Prm_1 != Num8) && (Prm_1 != Num16) && (Prm_1 != Num32))
        #error "OpenADC Parameters must be constant values"
        Exitm
    #endif
    #if((Prm_2 != Num8) && (Prm_2 != Num16) && (Prm_2 != Num32))
        #error "OpenADC Parameters must be constant values"
        Exitm
    #endif
    #if((Prm_3 != Num8) && (Prm_3 != Num16) && (Prm_3 != Num32))
        #error "OpenADC Parameters must be constant values"
        Exitm
    #endif
    #if((Prm_4 != Num8) && (Prm_4 != Num16) && (Prm_4 != Num32))
        #error "OpenADC Parameters must be constant values"
        Exitm
    #endif
    Num_SFR ((pConfig2 >> 1) & 0x7C),ADCON0     ' %01111100

    Num_SFR((pConfig3 & 0x0C) | (pConfig3 & 0x03)),ADCON1 'Positive and Negative Voltage Reference Configuration bits

    Num_SFR (pConfig & 128 | ((pConfig >> 4) & 0x07) | ((pConfig << 2) & 0x38)),ADCON2


    #if(pPortConfig <= 0x1F)    ' 0000000000011111
        ANSELB = 0
        ANSELC = 0
        Movlw (pPortConfig)
        ANSELA = WREG
    #endif

    #if((pPortConfig >= 0x0100) && (pPortConfig <= 0x3F00))
        ANSELA = 255
        Movlw (pPortConfig >> 8)
        ANSELB = WREG
        ANSELC = 0
    #endif

    #if((pPortConfig >= 0x010000) && (pPortConfig <= 0x3F0000))
        ANSELA = 255
        Movlw (pPortConfig >> 14)
        ANSELC = WREG
        ANSELB = 255
    #endif

    ADCON0bits_ADON = 1
'
' Dummy read, as per errata workaround
'
    ADCON0bits_GO_DONE = 1
    #ifdef WatchDog_Req
        Clrwdt
    #endif       
    Btfsc ADCON0bits_GO_DONE
    Bra($ - 2)
Endm

'-------------------------------------------------------------------------
' *** CloseADC Macro ***
'-------------------------------------------------------------------------

$define CloseADC()      '
    ADCON0bits_ADON = 0 '
    PIE1bits_ADIE = 0

'-------------------------------------------------------------------------
' *** SelChanConvADC Macro ***
'-------------------------------------------------------------------------

$define SelChanConvADC(pCH) _SelChanConvADC pCH

_SelChanConvADC Macro- pChannel
    #if(Prm_Count != 1)
        #error "SelChanConvADC must have 1 Parameter"
        Exitm
    #endif
    #if((Prm_1 != Num8) && (Prm_1 != Num16))
        #error "SelChanConvADC Parameter must be constant value"
        Exitm
    #endif
    Movlw $83
    Andwf ADCON0,W
    Iorlw ((pChannel >> 1) & 0x7C)
    Movwf ADCON0
    ADCON0bits_GO_DONE = 1
Endm

'-------------------------------------------------------------------------
' *** SetChanADC Macro ***
'-------------------------------------------------------------------------

$define SetChanADC(pCH) _SetChanADC pCH

_SetChanADC Macro- pChannel
    #if(Prm_Count != 1)
        #error "SetChanADC must have 1 Parameter"
        Exitm
    #endif
    #if((Prm_1 != Num8) && (Prm_1 != Num16))
        #error "SetChanADC Parameter must be constant value"
        Exitm
    #endif
    Num_SFR ((pChannel >> 1) & 0x7C),ADCON0
    Movlw $83
    Andwf ADCON0,W
    Iorwf ADCON0,F
Endm
'-------------------------------------------------------------------------
' *** BusyADC Macro ***
'-------------------------------------------------------------------------
$define BusyADC() _BusyADC

_BusyADC Macro- \ Byte

    #if(BusyADC_Return == 1)
        Movlw 0
        Btfsc ADCON0bits_GO_DONE
        Movlw 1
        #if(Return_Type == Byte)
            wreg_byte Return_Var
        #endif
        #if(Return_Type == Word)
            Wreg_Word Return_Var
        #endif
        #if(Return_Type == Dword)
            Wreg_Dword Return_Var
        #endif
        #if(Return_Type == Bit)
            Wreg_Bit Return_Var, Return_BitNum
        #endif
    #endif
Endm
'-------------------------------------------------------------------------
' *** ReadADC Macro ***
'-------------------------------------------------------------------------
$define ReadADC(pCH) _ReadADC pCH

_ReadADC Macro- pChannel \ Word
    #if(Prm_Count == 0)
        ADCON0bits_GO_DONE = 1
        #ifdef WatchDog_Req
            Clrwdt
        #endif
        Btfsc ADCON0bits_GO_DONE
        Bra($ - 2)
        
        #if(_ReadADC_Return == 1)
            #if(Return_Type == Bit)
                Byte_Bit ADRESH, Return_Var, Return_BitNum
            #endif
            #if(Return_Type == Byte)
                Byte_Byte ADRESH,Return_Var
            #endif
            #if(Return_Type == Word)
                #if(Return_Var != ADRESL)
                    Byte_Byte ADRESL,Return_Var
                    Byte_Byte ADRESH,Return_Var + 1
                #endif
            #endif
            #if(Return_Type == Dword)
                Byte_Byte ADRESL,Return_Var
                Byte_Byte ADRESH,Return_Var + 1
                Num_Word 0,Return_Var + 2
            #endif
        #endif
        Exitm
    #endif

    #if(Prm_Count > 1)
        #error "ReadADC must have only 1 Parameter"
        Exitm
    #endif

    #if((Prm_1 != Num8) && (Prm_1 != Num16))
        #error "ReadADC Parameter must be constant value"
        Exitm
    #endif
    Movlw $83
    Andwf ADCON0,W
    Iorlw ((pChannel >> 1) & 0x7C)
    Movwf ADCON0
    ADCON0bits_GO_DONE = 1
    #ifdef WatchDog_Req
        Clrwdt
    #endif
    Btfsc ADCON0bits_GO_DONE
    Bra($ - 2)
    #if(_ReadADC_Return == 1)
        #if(Return_Type == Bit)
            Byte_Bit ADRESH, Return_Var, Return_BitNum
        #endif
        #if(Return_Type == Byte)
            Byte_Byte ADRESH,Return_Var
        #endif
        #if(Return_Type == Word)
            #if(Return_Var != ADRESL)
                Byte_Byte ADRESL,Return_Var
                Byte_Byte ADRESH,Return_Var + 1
            #endif
        #endif
        #if(Return_Type == Dword)
            Byte_Byte ADRESL,Return_Var
            Byte_Byte ADRESH,Return_Var + 1
            Num_Word 0,Return_Var + 2
        #endif
    #endif
Endm

$endif

'$endif

