$ifndef _TimerMacroDef_
$define _TimerMacroDef_
'
' Timer libraries for Amicus18
'

' Interrupt bit mask to be 'anded' with the other configuration masks and
' passed as the 'pConfig' parameter to the 'TimerxOpen' macros.

$define TIMER_INT_OFF  %01111111  ' Interrupts disabled
$define TIMER_INT_ON   %11111111  ' Interrupts enabled

'-------- Timer 0 Declarations ---------------------
' Timer0 configuration masks -- to be 'anded' together

$define T0_16BIT       %10111111  ' Timer 0 is in 16 Bit mode
$define T0_8BIT        %11111111  ' Timer 0 is in 8 bit mode
$define T0_SOURCE_INT  %11011111  ' Internal clock source
$define T0_SOURCE_EXT  %11111111  ' External clock source
$define T0_EDGE_RISE   %11101111  ' External rising edge clocs timer
$define T0_EDGE_FALL   %11111111  ' External falling edge clocks timer
$define T0_PS_1_1      %11111111  ' Prescaler 1:1 (No Prescaler)
$define T0_PS_1_2      %11110000  '           1:2
$define T0_PS_1_4      %11110001  '           1:4
$define T0_PS_1_8      %11110010  '           1:8
$define T0_PS_1_16     %11110011  '           1:16
$define T0_PS_1_32     %11110100  '           1:32
$define T0_PS_1_64     %11110101  '           1:64
$define T0_PS_1_128    %11110110  '           1:128
$define T0_PS_1_256    %11110111  '           1:256

'-------- Timer 1 Declarations ---------------------
' Timer1 configuration masks -- to be 'anded' together

$define T1_8BIT_RW      %10111111   ' 8-bit mode
$define T1_16BIT_RW     %11111111   ' 16-bit mode
$define T1_PS_1_1       %11001111   ' 1:1 prescale value
$define T1_PS_1_2       %11011111   ' 1:2 prescale value
$define T1_PS_1_4       %11101111   ' 1:4 prescale value
$define T1_PS_1_8       %11111111   ' 1:8 prescale value
$define T1_OSC1EN_OFF   %11110111   ' Timer 1 oscillator enable off
$define T1_OSC1EN_ON    %11111111   ' Timer 1 oscillator enable on
$define T1_SYNC_EXT_ON  %11111011   ' Synchronise external clock input
$define T1_SYNC_EXT_OFF %11111111   ' Do not synchronise external clock input
$define T1_SOURCE_INT   %11111101   ' Internal clock source
$define T1_SOURCE_EXT   %11111111   ' External clock source

'-------- Timer 2 Declarations ---------------------
' Timer2 configuration masks -- to be 'anded' together

$define T2_POST_1_1    0x87  ' Postscaler 1:1
$define T2_POST_1_2    0x8F  ' Postscaler 1:2
$define T2_POST_1_3    %10010111  ' Postscaler 1:3
$define T2_POST_1_4    %10011111  ' Postscaler 1:4
$define T2_POST_1_5    %10100111  ' Postscaler 1:5
$define T2_POST_1_6    %10101111  ' Postscaler 1:6
$define T2_POST_1_7    %10110111  ' Postscaler 1:7
$define T2_POST_1_8    %10111111  ' Postscaler 1:8
$define T2_POST_1_9    %11000111  ' Postscaler 1:9
$define T2_POST_1_10   %11001111  ' Postscaler 1:10
$define T2_POST_1_11   %11010111  ' Postscaler 1:11
$define T2_POST_1_12   %11011111  ' Postscaler 1:12
$define T2_POST_1_13   %11100111  ' Postscaler 1:13
$define T2_POST_1_14   %11101111  ' Postscaler 1:14
$define T2_POST_1_15   %11110111  ' Postscaler 1:15
$define T2_POST_1_16   %11111111  ' Postscaler 1:16
$define T2_PS_1_1      %11111100  ' Prescale 1:1
$define T2_PS_1_4      %11111101  ' Prescale 1:4
$define T2_PS_1_16     %11111110  ' Prescale 1:16

'-------- Timer 3 Declarations ---------------------
' Timer3 configuration masks -- to be 'anded' together

$define T3_8BIT_RW      0xFE ' 8-bit mode
$define T3_16BIT_RW     0xFF ' 16-bit mode

$define T3_PS_1_1       0xCF ' 1:1 prescale value
$define T3_PS_1_2       0xDF ' 1:2 prescale value
$define T3_PS_1_4       0xEF ' 1:4 prescale value
$define T3_PS_1_8       0xFF ' 1:8 prescale value

$define T3_SYNC_EXT_ON  0xFB ' Synchronise external clock input
$define T3_SYNC_EXT_OFF 0xFF ' Do not synchronise external clock input

$define T3_SOURCE_INT   0xFD ' Internal clock source
$define T3_SOURCE_EXT   0xFF ' External clock source


$define T3_SOURCE_CCP   0xFF ' T3 is source for CCP
$define T1_CCP1_T3_CCP2 0xBF ' T1 is source for CCP1 and T3 is source for CCP2
$define T1_SOURCE_CCP   0xB7 ' T1 is source for CCP

$ifdef _Amicus18_Rev2 ' *** Declarations for 18F25K22
'-------- Timer 4 Declarations ---------------------
' Timer4 configuration masks -- to be 'anded' together

$define T4_POST_1_1    %10000111  ' Postscaler 1:1
$define T4_POST_1_2    %10001111  ' Postscaler 1:2
$define T4_POST_1_3    %10010111  ' Postscaler 1:3
$define T4_POST_1_4    %10011111  ' Postscaler 1:4
$define T4_POST_1_5    %10100111  ' Postscaler 1:5
$define T4_POST_1_6    %10101111  ' Postscaler 1:6
$define T4_POST_1_7    %10110111  ' Postscaler 1:7
$define T4_POST_1_8    %10111111  ' Postscaler 1:8
$define T4_POST_1_9    %11000111  ' Postscaler 1:9
$define T4_POST_1_10   %11001111  ' Postscaler 1:10
$define T4_POST_1_11   %11010111  ' Postscaler 1:11
$define T4_POST_1_12   %11011111  ' Postscaler 1:12
$define T4_POST_1_13   %11100111  ' Postscaler 1:13
$define T4_POST_1_14   %11101111  ' Postscaler 1:14
$define T4_POST_1_15   %11110111  ' Postscaler 1:15
$define T4_POST_1_16   %11111111  ' Postscaler 1:16
$define T4_PS_1_1      %11111100  ' Prescale 1:1
$define T4_PS_1_4      %11111101  ' Prescale 1:4
$define T4_PS_1_16     %11111111  ' Prescale 1:16

'-------- Timer 5 Declarations ---------------------
' Timer5 configuration masks -- to be 'anded' together

$define T5_SLP_EN               %11111111   ' Enable during sleep
$define T5_SLP_DIS              %01111111   ' Disable during sleep

$define T5_SP_EVNT_REN          %10111111   ' special event reset enable
$define T5_SP_EVNT_RDIS         %11111111   ' special event reset disable

$define T5_MD_SNGL_SHOT         %11111111   ' Single Shot Mode
$define T5_MD_CONT_COUNT        %11011111   ' Continous Count Mode

$define T5_PS_1_1               %11100111   ' Prescale 1:1
$define T5_PS_1_2               %11101111   ' Prescale 1:2
$define T5_PS_1_4               %11110111   ' Prescale 1:4
$define T5_PS_1_8               %11111111   ' Prescale 1:8

$define T5_EX_CLK_SYNC          %11111011   ' Ext. Clock Synchronization
$define T5_EX_CLK_NOSYNC        %11111111   ' Do Not synchronise

$define T5_CLK_EXTRN            %11111111   ' Clock source external
$define T5_CLK_INT              %11111101   ' Clock source internal

'-------- Timer 6 Declarations ---------------------
' Timer6 configuration masks -- to be 'anded' together

$define T6_POST_1_1    %10000111  'Timer6 Postscaler 1:1
$define T6_POST_1_2    %10001111  'Timer6 Postscaler 1:2
$define T6_POST_1_3    %10010111  'Timer6 Postscaler 1:3
$define T6_POST_1_4    %10011111  'Timer6 Postscaler 1:4
$define T6_POST_1_5    %10100111  'Timer6 Postscaler 1:5
$define T6_POST_1_6    %10101111  'Timer6 Postscaler 1:6
$define T6_POST_1_7    %10110111  'Timer6 Postscaler 1:7
$define T6_POST_1_8    %10111111  'Timer6 Postscaler 1:8
$define T6_POST_1_9    %11000111  'Timer6 Postscaler 1:9
$define T6_POST_1_10   %11001111  'Timer6 Postscaler 1:10
$define T6_POST_1_11   %11010111  'Timer6 Postscaler 1:11
$define T6_POST_1_12   %11011111  'Timer6 Postscaler 1:12
$define T6_POST_1_13   %11100111  'Timer6 Postscaler 1:13
$define T6_POST_1_14   %11101111  'Timer6 Postscaler 1:14
$define T6_POST_1_15   %11110111  'Timer6 Postscaler 1:15
$define T6_POST_1_16   %11111111  'Timer6 Postscaler 1:16
$define T6_PS_1_1      %11111100  'Timer6 Prescale 1:1
$define T6_PS_1_4      %11111101  'Timer6 Prescale 1:4
$define T6_PS_1_16     %11111111  'Timer6 Prescale 1:16


$define T34_SOURCE_CCP          %11111111   ' T3 and T4 are sources for CCP1 thru CCP5
$define T12_CCP12_T34_CCP345    %11110111   ' T1 and T2 are sources for CCP1 and CCP2 and T3 and T4 are sources for CCP3 thru CCP5
$define T12_CCP1_T34_CCP2345    %10111111   ' T1 and T2 are sources for CCP1 and T3 and T4 are sources for CCP2 thru CCP5
$define T12_SOURCE_CCP          %10110111   ' T1 and T2 are sources for CCP1 thru CCP5

$define T34_SOURCE_CCP12        %11111111   ' T3 and T4 are sources for CCP1 and CCP2
$define T12_CCP1_T34_CCP2       %11111101   ' T1 and T2 are sources for CCP1 and T3 and T4 are sources for CCP2
$define T12_SOURCE_CCP          %11111100   ' T1  and T2 is source for CCP1 and CCP2

$endif
(********************************************************************************)
'
' Timer0 Macros
'
' Create a word sized Timer0 for reading and writing
'
    Dim wTimer0M As TMR0L.Word
'---------------------------------------------------------------------------------
' Macro Name    : OpenTimer0
' Return Value  : None
' Parameters    : Bit definitions to configure Timer0
' Notes         : Reset the Timer0 regs to the POR state and configures the interrupt, clock source, edge and prescaler.
'               : The bit definitions for config can be found at the top of this file.
'
$define OpenTimer0(pConfig) _OpenTimer0 pConfig

_OpenTimer0 Macro pConfig
    #if(Prm_Count != 1)
        #error "OpenTimer0 Macro requires a parameter"
        Exitm
    #endif
    #if((Prm_1 != Num8) && (Prm_1 != Num16) && (Prm_1 != Num32))
        #error "OpenTimer0 Macro parameter must be a constant value"
        Exitm
    #endif

    #if((0x7F & pConfig) == 0)
        Clear T0CON
    #else
        Num_SFR (0x7F & pConfig),T0CON  ' Configure timer0, but don't start it yet
    #endif
    Clear TMR0H                      ' Reset Timer0 to 0x0000
    Clear TMR0L
    Clear INTCONbits_T0IF            ' Clear Timer0 overflow flag
    #if(pConfig & 0x80)              ' If interrupts enabled
        Set INTCONbits_T0IE          ' Enable Timer0 overflow interrupt
    #else
        Clear INTCONbits_T0IE        ' Disable Timer0 overflow interrupt
    #endif
    Set T0CONbits_TMR0ON
    Endm
'---------------------------------------------------------------------------------
' Macro Name    : CloseTimer0
' Return Value  : None
' Parameters    : None
' Description   : Disable the Timer0 interrupt and stop Timer0
'
$define CloseTimer0()       '
    Clear INTCONbits_TMR0IE '
    T0CON = $7F

'---------------------------------------------------------------------------------
' Macro Name    : ReadTimer0
' Return Value  : Timer0 value
' Parameters    : None
' Description   : Read a value from Timer0.
'
$define ReadTimer0() wTimer0M
'
'---------------------------------------------------------------------------------
' Macro Name    : WriteTimer0
' Return Value  : None
' Parameters    : Value to write to Timer0
' Description   : Write a value to Timer0
'
$define WriteTimer0(pTimerValue) wTimer0M = pTimerValue
(********************************************************************************)
'
' Timer1 Macros
'
' Create a word sized Timer1 for reading and writing
'
    Dim wTimer1M As TMR1L.Word

'---------------------------------------------------------------------------------
' Macro Name    : OpenTimer1
' Return Value  : None
' Parameters    : pConfig. Bit definitions to configure Timer1
' Notes         : Reset the Timer1 regs to the POR state and configures the interrupt, clock source, edge and prescaler.
'               : The bit definitions for config can be found at the top of this file.
'
$define OpenTimer1(pConfig) _OpenTimer1 pConfig

_OpenTimer1 Macro pConfig
    #if(Prm_Count != 1)
        #error "OpenTimer1 Macro requires a parameter"
        Exitm
    #endif
    #if((Prm_1 != Num8) && (Prm_1 != Num16) && (Prm_1 != Num32))
        #error "OpenTimer1 Macro parameter must be an unsigned constant value"
        Exitm
    #endif
    #if((0x7E & pConfig) == 0)
        Clear T1CON
    #else
        Num_SFR (0x7E & pConfig),T1CON ' Configure timer1, but don't start it yet
    #endif
    #if(pConfig & 0x40)             ' The 8/16 selection bit
        Set T1CONbits_RD16          ' Enabled
    #else
        Clear T1CONbits_RD16        ' Disabled
    #endif
    Clear TMR1H                     ' Reset Timer1 to 0
    Clear TMR1L
    Clear PIR1bits_TMR1IF           ' Clear Timer1 overflow flag
    #if(pConfig & 0x80)             ' If interrupts enabled
        Set PIE1bits_TMR1IE         ' Enable Timer1 overflow interrupt
    #else                           ' Otherwise...
        Clear PIE1bits_TMR1IE       ' Disable Timer1 overflow interrupt
    #endif
    Set T1CONbits_TMR1ON
    Endm

'---------------------------------------------------------------------------------
' Macro Name     : CloseTimer1
' Return Value   : None
' Parameters     : None
' Description    : Disable the Timer1 interrupt and stop Timer0
'
$define CloseTimer1()      '
    Clear T1CONbits_TMR1ON '
    Clear PIE1bits_TMR1IE

'---------------------------------------------------------------------------------
' Macro Name     : ReadTimer1
' Return Value   : Timer1 value
' Parameters     : None
' Description    : Read a value from Timer1
'
$define ReadTimer1() wTimer1M

'---------------------------------------------------------------------------------
' Macro Name     : WriteTimer1
' Return Value   : None
' Parameters     : Value to write to Timer1
' Description    : Write a value to Timer1
'
$define WriteTimer1(pTimerValue) wTimer1M = pTimerValue

(********************************************************************************)
'
' Timer2 Macros
'
'---------------------------------------------------------------------------------
' Macro Name    : OpenTimer2
' Return Value  : None
' Parameters    : pConfig. Bit definitions to configure Timer2
' Notes         : Reset the Timer2 regs to the POR state and configures the interrupt, clock source, edge and prescaler.
'               : The bit definitions for config can be found at the top of this file.
'
$define OpenTimer2(pConfig) _OpenTimer2 pConfig

_OpenTimer2 Macro pConfig
    #if(Prm_Count != 1)
        #error "OpenTimer2 Macro requires a parameter"
        Exitm
    #endif
    #if((Prm_1 != Num8) && (Prm_1 != Num16) && (Prm_1 != Num32))
        #error "OpenTimer2 Macro parameter must be an unsigned constant value"
        Exitm
    #endif
    #if((0xFB & pConfig) == 0)
        Clear T2CON
    #else
        Num_SFR (0xFB & pConfig),T2CON  ' Configure Timer2, but don't start it yet
    #endif
    Clear TMR2                      ' Reset Timer2 to 0x00
    Clear PIR1bits_TMR2IF           ' Clear Timer2 overflow flag
    #if(pConfig & 0x80)             ' If interrupts enabled
        Set PIE1bits_TMR2IE         ' Enable Timer2 overflow interrupt
    #else
        Clear PIE1bits_TMR2IE       ' Disable Timer2 overflow interrupt
    #endif
    Set T2CONbits_TMR2ON            ' Enable Timer2
    Endm
'---------------------------------------------------------------------------------
' Macro Name    : CloseTimer2
' Return Value  : None
' Parameters    : None
' Description   : Disable the Timer2 interrupt and stop Timer2
'
$define CloseTimer2()      '
    Clear T2CONbits_TMR2ON '
    Clear PIE1bits_TMR2IE

'---------------------------------------------------------------------------------
' Macro Name    : ReadTimer2
' Return Value  : Timer2 value
' Parameters    : None
' Description   : Read a value from Timer2.
'
$define ReadTimer2() TMR2

'---------------------------------------------------------------------------------
' Macro Name      : WriteTimer2
' Return Value    : None
' Parameters      : Value to write to Timer2
' Description     : Write a value to Timer2
'
$define WriteTimer2(pTimerValue) TMR2 = pTimerValue

(********************************************************************************)
'
' Timer3 Macros
'
    Dim wTimer3M As TMR3L.Word
'---------------------------------------------------------------------------------
' Macro Name    : T3_OSC1EN_ON
' Overview      : Sets the T1OSCEN bit in the T1CON
' Parameters    : None
' Remarks       : None.
$define T3_OSC1EN_ON() T1CONbits_T1OSCEN = 1      ' Timer 3 oscillator enables

'---------------------------------------------------------------------------------
' Macro name    : T3_OSC1EN_OFF
' Overview      : Clears the T1OSCEN bit in the T1CON
' Parameters    : None
' Remarks       : None
$define T3_OSC1EN_OFF() T1CONbits_T1OSCEN = 0     ' Timer 3 oscillator Disables

'---------------------------------------------------------------------------------
' Macro Name    : OpenTimer3
' Return Value  : None
' Parameters    : pConfig: bit definitions to configure Timer3
' Description   : This routine first resets the Timer3 regs to the POR state and then configures the interrupt, clock source.
' Notes         : The bit definitions for config can be found at the top of this file.
'
$define OpenTimer3(pConfig) _OpenTimer3 pConfig

_OpenTimer3 Macro pConfig
    #if(Prm_Count != 1)
        #error "OpenTimer3 Macro requires a parameter"
        Exitm
    #endif
    #if((Prm_1 != Num8) && (Prm_1 != Num16) && (Prm_1 != Num32))
        #error "OpenTimer3 Macro parameter must be an unsigned constant value"
        Exitm
    #endif
    #if((0x7E & pConfig) == 0)
        Clear T3CON
    #else
        Num_SFR (0x7E & pConfig),T3CON ' Set everything except 8/16 mode, and don't start timer yet
    #endif
    Clear TMR3H                       ' Clear timer3-related registers
    Clear TMR3L
    Clear PIR2bits_TMR3IF
    #if(pConfig & 0x80)               ' Interrupt on/off
        Set PIE2bits_TMR3IE
    #else
        Clear PIE2bits_TMR3IE
    #endif
    #if(pConfig & 2)
        Set T1CONbits_T1OSCEN         ' Enable Timer 1 Oscillator enable (200KHz max freq)
    #endif
    #if(pConfig & ~T3_8BIT_RW)        ' Select between 8-bit and 16-bit modes
        Set T3CONbits_RD16
    #else
        Clear T3CONbits_RD16
    #endif
    Set T3CONbits_TMR3ON              ' Turn on Timer3
    Endm

'---------------------------------------------------------------------------------
' Macro Name    : CloseTimer3
' Return Value  : None
' Parameters    : None
' Description   : Disable the Timer3 interrupt and stop Timer0
'
$define CloseTimer3()      '
    Clear T3CONbits_TMR3ON '
    Clear PIE2bits_TMR3IE

'---------------------------------------------------------------------------------
' Macro Name    : ReadTimer3
' Return Value  : Timer3 value
' Parameters    : None
' Description   : Read a value from Timer3.
'
$define ReadTimer3() wTimer3M

'---------------------------------------------------------------------------------
' Macro Name    : WriteTimer3
' Return Value  : None
' Parameters    : Value to write to Timer3
' Description   : Write a value to Timer3
'
$define WriteTimer3(pTimerValue) wTimer3M = pTimerValue

$endif
