$ifndef __PID__
$define __PID__
'
' Experimental Amicus18 integer only PID routine
' Written by Les Johnson (14-11-11)
' For 18F devices only
'
' Signed 16-bit divide variable aliases
'    
    Dim Sdiv_bCounter As PRODH
    Dim Sdiv_bSign As PRODL
    Dim PP0 As Byte System
    Dim PP0H As Byte System
    Dim PP1 As Byte System
    Dim PP1H As Byte System
    Dim Sdiv_wRemainder As Word System
    Dim Sdiv_wArg1 As PP0.Word
    Dim Sdiv_wArg2 As PP1.Word
'
' PID variables
'    
    Dim PID_wInput As Word                      ' Input to the PID routine    
    Dim PID_wRequired As Word                   ' Optimum value to acheive   
    Dim PID_wOutput As Word                     ' Output of the PID
    
    Dim PID_wCorrectionMax As Word              ' Maximum correction to be applied 
    Dim PID_wOutputMax As Word                  ' The maximum output value allowed
    Dim PID_wOutputMin As Word                  ' The minimum output value allowed
    Dim PID_wPgain As Word                      ' Response factor for P
    Dim PID_wIgain As Word                      ' Response factor for I
    Dim PID_wDgain As Word                      ' Response factor for D 
    
    Dim PID_wPreviousInput As Word              ' Previous input of the PID
    Dim PID_wError As Word                      ' Error observed
    Dim PID_wCorrection As Word                 ' Internal correction to be applied    
    Dim PID_wErrorSum As Word                   ' Sum of errors observed
    Dim PID_wErrorMax As Word                   ' Max error sum allowed      
    Dim PID_wTemp As Word                       ' Temporary word variable
    
'-----------------------------------------------------------------------------------------------    
    GoTo _PID_Main                              ' Jump over the subroutines
'-----------------------------------------------------------------------------------------------
' Divide Signed Word by Signed Word to give a Signed Word output
' Input     : Sdiv_wArg1 holds the signed 16-bit dividend
'           : Sdiv_wArg2 holds the signed 16-bit divisor
' Output    : Sdiv_wArg1 holds the signed 16-bit result
'           : Sdiv_wRemainder holds the remainder of the division
' Notes     : None
'
$define Div_SWord_SWord(pDividend, pDivisor) _divS16_16 pDividend, pDivisor

_divS16_16 Macro- pDividend, pDivisor \Word
    #if((Prm_1 == Num8) || (Prm_1 == Num16) || (Prm_1 == Num32) || (Prm_1 == SNum8) || (Prm_1 == SNum16) || (Prm_1 == SNum32))    
        Num_Word pDividend, Sdiv_wArg1
    #endif
    #if(Prm_1 == Byte)
        Byte_Word pDividend, Sdiv_wArg1    
    #endif
    #if((Prm_1 == Word) || (Prm_1 == Dword))
        Word_Word pDividend, Sdiv_wArg1   
    #endif
    
    #if((Prm_2 == Num8) || (Prm_2 == Num16) || (Prm_2 == Num32) || (Prm_2 == SNum8) || (Prm_2 == SNum16) || (Prm_2 == SNum32))    
        Num_Word pDivisor, Sdiv_wArg2
    #endif
    #if(Prm_2 == Byte)
        Byte_Word pDivisor, Sdiv_wArg2
    #endif
    #if((Prm_2 == Word) || (Prm_2 == Dword))
        Word_Word pDivisor, Sdiv_wArg2
    #endif
    
    GoSub divS16_16
    Return_Word Sdiv_wArg1 
    Endm
    
#ifMacro- _divS16_16        ' Only include this code if the macro is used within the program
divS16_16:
    Clear Sdiv_wRemainder
    Sdiv_bCounter = 17
    Movf Sdiv_wArg2H,w 
    Xorwf Sdiv_wArg1H,w
    Movwf Sdiv_bSign
    Btfss Sdiv_wArg1H,7 
    Bra divS16_16skip
    Sdiv_wArg1 = -Sdiv_wArg1               
divS16_16skip:    
    Btfss Sdiv_wArg2H,7 
    Bra divS16_16Enty
    Sdiv_wArg2 = -Sdiv_wArg2
    Bra divS16_16Enty                       
divS16_16l3:    
    Rol Sdiv_wRemainder
    Movf Sdiv_wArg2,w  
    Subwf Sdiv_wRemainder,w
    Movf Sdiv_wArg2H,w 
    Subwfb Sdiv_wRemainderH,w              
    Bnc divS16_16Enty 
    Movwf Sdiv_wRemainderH    
    Movf Sdiv_wArg2,w  
    Subwf Sdiv_wRemainder,f
    Bsf STATUS,C                  
divS16_16Enty:    
    Rol Sdiv_wArg1
    Decfsz Sdiv_bCounter,f 
    Bra divS16_16l3    
    Btfss Sdiv_bSign,7
    Bra divS16_16Exit
    Sdiv_wArg1 = -Sdiv_wArg1
divS16_16Exit:
    Return
#endIfMacro-

'------------------------------------------------------------------------
' Experimental Integer PID routine
' Input     : PID_wInput holds the measurement from a sensor (-32768 to +32767)
'           : PID_wRequired holds the optimum value to acheive
'           : PID_wCorrectionMax holds the maximum correction to apply
'           : PID_wErrorMax holds the maximum error allowed
'           : PID_wPgain holds the Proportional gain (-32768 to +32767)
'           : PID_wIgain holds the Integral gain (-32768 To +32767)
'           : PID_wDgain holds the Derivative gain (-32768 To +32767)
'           : PID_wOutputMax holds the maximum output allowed
'           : PID_wOutputMin holds the minimum output allowed
' Output    : PID_wOutput holds the result of the PID calculation
' Notes     : Uses a pre-processor macro for ease of use
'
$define PID_Compute(pInput, pRequired, pResult) '
    PID_wInput = pInput         '
    PID_wRequired = pRequired   '
    GoSub _PID                  '
    pResult = PID_wOutput
    
_PID:
    PID_wError = PID_wInput - PID_wRequired                             ' Find the error between what's required and what's just been input
    PID_wCorrection = PID_wError * PID_wPgain                           ' P factor
    PID_wErrorSum = PID_wErrorSum + PID_wError                          ' Sum the error value
    PID_wTemp = PID_wErrorSum                                           ' \
    If PID_wTemp.15 = 1 Then                                            ' | Make it a positive value for testing
        PID_wTemp = -PID_wTemp                                          ' |
    EndIf                                                               ' /
    If PID_wTemp > PID_wErrorMax Then                                   ' Is the accumulative error greater than that allowed?
        If PID_wErrorSum.15 = 1 Then                                    ' Yes. So was it originally a negative value?
            PID_wErrorSum = -PID_wErrorMax                              ' Yes. So make it negative maximum
        Else                                                            ' Otherwise...  
            PID_wErrorSum = PID_wErrorMax                               ' Make it positive maximum
        EndIf
    EndIf
    PID_wCorrection = PID_wCorrection + (PID_wErrorSum * PID_wIgain)    ' I factor
    PID_wTemp = ((PID_wPreviousInput - PID_wInput) * PID_wDgain)        ' \
    PID_wCorrection = PID_wCorrection + PID_wTemp                       ' / D factor
    PID_wPreviousInput = PID_wInput                                     ' Save the current measurement
    PID_wCorrection = Div_SWord_SWord(PID_wCorrection, 64)              ' Signed 16-bit divide for scaling        
    PID_wTemp = PID_wCorrection                                         ' \
    If PID_wTemp.15 = 1 Then                                            ' | Make it a positive value for testing
        PID_wTemp = -PID_wTemp                                          ' |
    EndIf                                                               ' /
    If PID_wTemp > PID_wCorrectionMax Then                              ' Is the correction greater than that allowed?
        If PID_wCorrection.15 = 1 Then                                  ' Yes. So was it originally a negative value?
            PID_wCorrection = -PID_wCorrectionMax                       ' Yes. So make it maximum negative
        Else                                                            ' Otherwise...
            PID_wCorrection = PID_wCorrectionMax                        ' Make it positive maximum
        EndIf
    EndIf
    PID_wOutput = PID_wOutput - PID_wCorrection                         ' Subtract the correction from the output result
    If PID_wOutput > PID_wOutputMax Then                                ' Scale maximum output
        PID_wOutput = PID_wOutputMax   
    ElseIf PID_wOutput <= PID_wOutputMin Then                           ' Scale minimum output
        PID_wOutput = PID_wOutputMin  
    EndIf
    Return
'------------------------------------------------------------------------
' Initialise the PID routine
' Input     : PID_wOutputMax =  Maximum output value
'           : PID_wOutputMin =  Minimum output value
'           : PID_wCorrectionMax =  Maximum correction to be applied
'           : PID_wErrorMax =  Maximum error sum allowed
'           : PID_wPgain =  Response factor for P 
'           : PID_wIgain =  Response factor for I
'           : PID_wDgain =  Response factor for D 
' Output    : None
' Notes     : Must be used at least once before the PID_Compute macro is used
'           : Uses a pre-processor macro for ease of use
'
$define PID_Setup(pKP, pKI, pKD, pMaxCorrection, pMaxError, pMinOutput, pMaxOutput) '
    PID_wOutputMax = pMaxOutput          '
    PID_wOutputMin = pMinOutput          '
    PID_wCorrectionMax = pMaxCorrection  '
    PID_wErrorMax = pMaxError            '
    PID_wPgain = pKP                     '
    PID_wIgain = pKI                     '
    PID_wDgain = pKD                     '
    PID_wPreviousInput = 0               '
    PID_wErrorSum = 0
    
'------------------------------------------------------------------------
_PID_Main:

$endif
