
$define _TimerMacroDef_
$define TIMER_INT_ON   %11111111  ' Interrupts enabled

''-------- Timer 0 Declarations ---------------------
'' Timer0 configuration masks -- to be 'anded' together

'$define T0_16BIT       %10111111  ' Timer 0 is in 16 Bit mode
'$define T0_8BIT        %11111111  ' Timer 0 is in 8 bit mode
'$define T0_SOURCE_INT  %11011111  ' Internal clock source
'$define T0_SOURCE_EXT  %11111111  ' External clock source
'$define T0_EDGE_RISE   %11101111  ' External rising edge clocs timer
'$define T0_EDGE_FALL   %11111111  ' External falling edge clocks timer
'$define T0_PS_1_1      %11111111  ' Prescaler 1:1 (No Prescaler)
'$define T0_PS_1_2      %11110000  '           1:2
'$define T0_PS_1_4      %11110001  '           1:4
'$define T0_PS_1_8      %11110010  '           1:8
'$define T0_PS_1_16     %11110011  '           1:16
'$define T0_PS_1_32     %11110100  '           1:32
'$define T0_PS_1_64     %11110101  '           1:64
'$define T0_PS_1_128    %11110110  '           1:128
'$define T0_PS_1_256    %11110111  '           1:256

'(********************************************************************************)
''
'' Timer0 Macros
''
'' Create a word sized Timer0 for reading and writing
''
'    Dim wTimer0M As TMR0L.Word
''---------------------------------------------------------------------------------
'' Macro Name    : OpenTimer0
'' Return Value  : None
'' Parameters    : Bit definitions to configure Timer0
'' Notes         : Reset the Timer0 regs to the POR state and configures the interrupt, clock source, edge and prescaler.
''               : The bit definitions for config can be found at the top of this file.
''
'$define OpenTimer0(pConfig) _OpenTimer0 pConfig

'_OpenTimer0 Macro pConfig
'    #if(Prm_Count != 1)
'        #error "OpenTimer0 Macro requires a parameter"
'        Exitm
'    #endif
'    #if((Prm_1 != Num8) && (Prm_1 != Num16) && (Prm_1 != Num32))
'        #error "OpenTimer0 Macro parameter must be a constant value"
'        Exitm
'    #endif

'    #if((0x7F & pConfig) == 0)
'        Clear T0CON
'    #else
'        Num_SFR (0x7F & pConfig),T0CON  ' Configure timer0, but don't start it yet
'    #endif
'    Clear TMR0H                      ' Reset Timer0 to 0x0000
'    Clear TMR0L
'    Clear INTCONbits_T0IF            ' Clear Timer0 overflow flag
'    #if(pConfig & 0x80)              ' If interrupts enabled
        Set INTCONbits_T0IE          ' Enable Timer0 overflow interrupt
    #else
        Clear INTCONbits_T0IE        ' Disable Timer0 overflow interrupt
    #endif
    Set T0CONbits_TMR0ON
   Endm
'---------------------------------------------------------------------------------
' Macro Name    : CloseTimer0
' Return Value  : None
' Parameters    : None
' Description   : Disable the Timer0 interrupt and stop Timer0
'
$define CloseTimer0()       '
    Clear INTCONbits_TMR0IE '
    T0CON = $7F

'---------------------------------------------------------------------------------
' Macro Name    : ReadTimer0
' Return Value  : Timer0 value
' Parameters    : None
' Description   : Read a value from Timer0.
'
$define ReadTimer0() wTimer0M
'
'---------------------------------------------------------------------------------
' Macro Name    : WriteTimer0
' Return Value  : None
' Parameters    : Value to write to Timer0
' Description   : Write a value to Timer0
'
$define WriteTimer0(pTimerValue) wTimer0M = pTimerValue


  OpenTimer0(TIMER_INT_ON  & T0_8BIT & T0_SOURCE_INT & T0_PS_1_256)

                                                                   
